;; Generated package description from standard-themes.el  -*- mode: lisp-data; no-byte-compile: t -*-
(define-package "standard-themes" "3.0.0" "Like the default theme but more consistent" '((emacs "28.1") (modus-themes "5.0.0")) :commit "d62c356289178062dec9e944cfafb790a77fe587" :authors '(("Protesilaos Stavrou" . "info@protesilaos.com")) :maintainer '("Protesilaos Stavrou" . "info@protesilaos.com") :keywords '("faces" "theme" "accessibility") :url "https://github.com/protesilaos/standard-themes")
